/**

  The Initial Developer of the Original Code is
  Matthieu  - http://www.programmation-facile.com/
  Portions created by the Initial Developer are Copyright (C) 2013
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

 */



/**
 * Le Framework de jeux JavaScript Phaser
 *
Phaser v2.0.5 - http://phaser.io - @photonstorm - (c) 2014 Photon Storm Ltd. 
https://github.com/photonstorm/phaser
http://examples.phaser.io/
 */



/**
 * Les variables du jeu
 * 
 */

var game;

var cursors;

var player;
var wallsBitmap;
var floor;

// gestion de la zone de lumière
var lightAngle = Math.PI/4;
var numberOfRays = 20;
var rayLength = 100;
var maskGraphics;

// la taille du jeu
var gameWidth = 640;
var gameHeight = 480;


document.addEventListener("DOMContentLoaded", Main, false); // appel au chargement de la page


/**
 * Fonction principale
 * Appelée au chargement de la page
 * 
 */
function Main()
{
    console.log("Main");

    // creation de la zone de jeu - API Canvas
    game = new Phaser.Game(gameWidth, gameHeight, Phaser.AUTO, "idGameDiv", {preload:onPreload, create:onCreate, update:onUpdate});

}


/**
 * Préchargement des éléments du jeu
 * 
 * @return {[type]} [description]
 */
function onPreload()
{ 
    game.stage.backgroundColor = '#151515';// la couleur de fond du jeu
    game.stage.backgroundColor = '#000000';// la couleur de fond du jeu

    game.load.image("sol","assets/floor.jpg");// chargement des images du jeu
    game.load.image("murs","assets/walls.png");
    game.load.image("player","assets/player.gif");
}



/**
 * Création de la scène du jeu,
 * mise en place des graphismes...
 * 
 * @return {[type]} [description]
 */
function onCreate() 
{   
    // ajoute le sol
    floor = game.add.sprite(0, 0, "sol");

    // ajoute le mur
    wallsBitmap = game.make.bitmapData(gameWidth, gameHeight);
    wallsBitmap.draw("murs");
    wallsBitmap.update();
    game.add.sprite(0,0, wallsBitmap);

    // ajoute le joueur
    player = game.add.sprite(80, 80, "player");
    player.anchor.setTo(0.5, 0.5);

    cursors = game.input.keyboard.createCursorKeys();// capture les actions sur les touches directionnelles du clavier

    // création d'un masque, pour la lumière
    maskGraphics = this.game.add.graphics(0, 0);
    floor.mask=maskGraphics;
}



/**
 * Appelée en continu pendant le jeu
 * 
 * @return {[type]} [description]
 */
function onUpdate() 
{
    var xSpeed = 0;
    var ySpeed = 0;
    var nSpeed = 2;// vitesse de déplacement du joueur

    if(cursors.up.isDown)
        ySpeed -= nSpeed;
    
    if(cursors.down.isDown)
        ySpeed += nSpeed;
    
    if(cursors.left.isDown)
        xSpeed -= nSpeed;
    
    if(cursors.right.isDown)
        xSpeed += nSpeed;
    

    // est ce qu'il y a un mur ?
    var color = wallsBitmap.getPixel32(player.x+xSpeed+player.width/2,player.y+ySpeed+player.height/2);
    color+= wallsBitmap.getPixel32(player.x+xSpeed-player.width/2,player.y+ySpeed+player.height/2);
    color+=wallsBitmap.getPixel32(player.x+xSpeed-player.width/2,player.y+ySpeed-player.height/2)
    color+=wallsBitmap.getPixel32(player.x+xSpeed+player.width/2,player.y+ySpeed-player.height/2)
    
    // déplacement du joueur seulement si c'est possible (pas de mur)
    if(color==0)
    {
        player.x+=xSpeed;
        player.y+=ySpeed;
    }       


    // affichage de la zone éclairée
    var mouseAngle = Math.atan2( player.y-game.input.y, player.x-game.input.x);
    maskGraphics.clear();
    maskGraphics.lineStyle(2, 0xffffff, 1);
    maskGraphics.beginFill(0xffff00);
    maskGraphics.moveTo(player.x,player.y); 

    for(var i = 0; i<numberOfRays; i++)
    {    
        var rayAngle = mouseAngle - (lightAngle/2)+(lightAngle/numberOfRays)*i
        var lastX = player.x;
        var lastY = player.y;

        for(var j= 1; j<=rayLength; j+=1)
        {
            var landingX = Math.round(player.x-(2*j)*Math.cos(rayAngle));
            var landingY = Math.round(player.y-(2*j)*Math.sin(rayAngle));

            if(wallsBitmap.getPixel32(landingX,landingY)==0)
            {
                lastX = landingX;
                lastY = landingY;   
            }

            else
            {
                maskGraphics.lineTo(lastX,lastY);
                break;
            }
        }

        maskGraphics.lineTo(lastX,lastY);
    }

    maskGraphics.lineTo(player.x,player.y); 
    maskGraphics.endFill();

    floor.alpha = 0.5+Math.random()*0.5;// effet de clignotement de la lumière
}



